#!/usr/bin/perl -w
# Given a table with scaffoldId, pos, strand, and data field(s),
# add on gene information (locusId, fraction) before the data field(s).
use strict;
use lib "feba/lib";
use FindGene; # for LocationToGene()

sub ReadTable($*); # filename and list of required fields => list of hashes, each with field->value

{
    die "Usage: AddGeneInfo.pl data_table gene_table > output\n" unless @ARGV==2;
    my ($datafile, $genesfile) = @ARGV;

    my @genes = &ReadTable($genesfile, qw(locusId scaffoldId begin end strand));

    my %genesSorted = (); # scaffold to list of genes sorted by begin
    foreach my $gene (@genes) {
	push @{ $genesSorted{$gene->{scaffoldId}} }, $gene;
    }
    foreach my $scaffold (keys %genesSorted) {
	my @sorted = sort { $a->{begin} <=> $b->{begin} } @{ $genesSorted{$scaffold} };
	$genesSorted{$scaffold} = \@sorted;
    }

    open(IN, "<", $datafile) || die "Cannot read $datafile";
    my $nLine = 0;
    my $nGenic = 0;
    while(<IN>) {
	chomp;
	my @F = split /\t/, $_, -1;
	die "Not enough columns: @F" unless @F >= 3;
	my $scaffold = shift @F;
	my $pos = shift @F;
	my $strand = shift @F;
	my ($locusId, $f) = &LocationToGene($scaffold, $pos, \%genesSorted);
	print join("\t", $scaffold, $pos, $strand, $locusId, $f, @F)."\n";
	$nLine++;
	$nGenic++ if $locusId ne "";
    }
    close(IN) || die "Error reading $datafile";
    print STDERR "Processed $nLine lines from $datafile, $nGenic in genes\n";
}

sub ReadTable($*) {
    my ($filename,@required) = @_;
    open(IN, "<", $filename) || die "Cannot read $filename";
    my $headerLine = <IN>;
    chomp $headerLine;
    my @cols = split /\t/, $headerLine;
    my %cols = map { $cols[$_] => $_ } (0..(scalar(@cols)-1));
    foreach my $field (@required) {
	die "No field $field in $filename" unless exists $cols{$field};
    }
    my @rows = ();
    while(my $line = <IN>) {
	chomp $line;
	my @F = split /\t/, $line, -1;
	die "Wrong number of columns in:\n$line\nin $filename"
	    unless scalar(@F) == scalar(@cols);
	my %row = map { $cols[$_] => $F[$_] } (0..(scalar(@cols)-1));
	push @rows, \%row;
    }
    close(IN) || die "Error reading $filename";
    return @rows;
}
