#!/usr/bin/perl
use strict;
use Getopt::Long;

{
    my $skip = 1;
    my $column = 1;
    my $name = undef;
    my $sortByName = 0;
    my $sum = undef;
    my $sumCol = undef;
    my $usage = "Usage: tabulate.pl [-skip 1] [-column 1 |-name Name] [-sortByName 1] [-sum name] < input > output\n"
	. "   Tabulates a column of a tab-delimited dataset, or, if -sum name or -sum column is used,\n"
	. "   sums that column\n";
    die $usage unless GetOptions('skip=i'=>\$skip, 'name=s'=> \$name, 'column=i'=>\$column,
				 'sortByName=i'=>\$sortByName, 'sum=s'=>\$sum)
	&& ($column==1 || !defined $name) && @ARGV==0;
    $column--;

    $column = undef if defined $name;

    while($skip > 0) {
	$_ = <STDIN>;
	chomp;
	if (defined $name && !defined $column) {
	    my @names = split /\t/,$_;
	    my @index = grep {$names[$_] eq $name} (0..$#names);
	    if (@index > 0) {
		$column = $index[0];
	    } else {
		die "Cannot find column named $name";
	    }
	    if (defined $sum && $sum !~ m/^[0-9]+$/) {
		my @sumindex = grep {$names[$_] eq $sum} (0..$#names);
		if (@sumindex > 0) {
		    $sumCol = $sumindex[0];
		} else {
		    die "Cannot find column named $sum";
		}
	    }
	}
	$skip--;
    }

    if (defined $sum && $sum =~ /^[0-9]+$/) {
	$sumCol = $sum-1;
    }

    my %counts = ();
    while(<STDIN>) {
	chomp;
	my @F = split /\t/,$_;
	my $d = $F[$column];
	$counts{$d} = 0 unless exists $counts{$d};
	if (defined $sumCol) {
	    $counts{$d} += $F[$sumCol];
	} else {
	    $counts{$d}++;
	}
    }
    print join("\t", defined $name ? $name : "Var1",
	       defined $sumCol ? "Sum$sum" : "Freq",
	       defined $sumCol ? "Frac$sum" : "Frac")."\n";
    my @keys = keys(%counts);
    if ($sortByName) {
	@keys = sort @keys;
    } else {
	@keys = sort { $counts{$b} <=> $counts{$a} } @keys;
    }
    my $tot = 0;
    foreach my $value (values %counts) {
	$tot += $value;
    }
    foreach(@keys) {
	print join("\t",$_,$counts{$_}, $counts{$_}/$tot)."\n";
    }
}
